﻿Option Explicit On
Imports System
'Imports System.IO.Path
Imports System.IO
Imports System.Drawing.Imaging



Public Class RTI_Processor


    Dim Image_Filename, Image_Extension, Image_Path As String
    Dim Storage_Directory, RTI_Directory As String
    Dim Total_images As Integer 'Count of images in directory, compared with image count in lps file
    Dim lps_load(257) As String 'Lines of lps file loaded in
    Dim Application_Path As String
    Dim First_load As Integer = 0 'Check for first processed file; one Message Box shows up the first time, disappears for subsequent runs
    Dim Grayscale_file As String = "" 'Name of grayscale JPG file used for color correction



    'Determines the data folder
    Private Sub cmdFolder_Click(sender As Object, e As EventArgs) Handles cmdFolder.Click
        Dim Batch_query As Integer

        If dlgDataFolder.ShowDialog() = Windows.Forms.DialogResult.OK Then
            Storage_Directory = dlgDataFolder.SelectedPath
            If Not Storage_Directory.EndsWith("\") Then Storage_Directory = Storage_Directory & "\"
            If Storage_Directory.Contains(" ") Then
                MsgBox("No spaces allowed")
                Storage_Directory = ""
                cmdFolder.PerformClick()
            End If
            lblFolderPath.Text = Storage_Directory
            'master_batch.bat is a file that contains postponed processing instructions. If it exists from a previous processing session, you have the option
            'to delete it if you've already run it to process files. If you haven't run it, keep it, and the postponed processing from this session will be appended.
            If My.Computer.FileSystem.FileExists(Storage_Directory & "master_batch.bat") Then
                Batch_query = MsgBox("Delete existing master process batch file?", vbYesNo)
                If Batch_query = vbYes Then Kill(Storage_Directory & "master_batch.bat")
            End If
        End If

    End Sub
    'Select one image from the folder that has all the RTI images; all of them will be copied into another folder for processing.
    Private Sub cmdOpen_Image_Click(sender As Object, e As EventArgs) Handles cmdOpen_Image.Click
        Dim i As Integer
        Dim foundFile As String
        'If no data folder was specified, you'll need to do that first.
        If Storage_Directory = "\" Or Storage_Directory = "" Then
            MsgBox("Looks like you didn't correctly select the storage directory, since it's showing up as blank or just a ""\""; try again.")
            Exit Sub
        End If
        If txtObjectID.Text = "" Then
            MsgBox("Please enter an Object ID first.")
            Exit Sub
        End If

        'Check to see if RTI_Directory already exists for that object ID
        RTI_Directory = Storage_Directory & txtObjectID.Text & "_RTI\"
        If My.Computer.FileSystem.DirectoryExists(RTI_Directory) Then
            MsgBox("There's already a directory with the name """ & RTI_Directory & """. Please choose a different Object ID name.")
            Exit Sub
        End If

        'Select one image from directory
        If dlgCommonDialog.ShowDialog() = Windows.Forms.DialogResult.OK Then
            Image_Filename = dlgCommonDialog.FileName
            Image_Path = System.IO.Path.GetDirectoryName(Image_Filename)
            If Not Image_Path.EndsWith("\") Then Image_Path = Image_Path & "\"
            Image_Extension = "*" & System.IO.Path.GetExtension(Image_Filename)


            'Check to make sure number of image files matches number in lps file
            i = 0
            For Each foundFile In My.Computer.FileSystem.GetFiles(Image_Path, Microsoft.VisualBasic.FileIO.SearchOption.SearchTopLevelOnly, Image_Extension)
                i += 1
            Next

            If i <> CInt(lps_load(0)) Then
                MsgBox("Number of image files in directory does not match number in lps file.")
                Exit Sub
            End If

            My.Computer.FileSystem.CreateDirectory(RTI_Directory)
            My.Computer.FileSystem.CreateDirectory(RTI_Directory & "jpeg-exports\")
            ' Following section pulls in all JPG filenames, copies them to RTI_Directory\jpeg-exports with new name

            'TIFF and BMP files will be converted to JPG format and saved in RTI_Directory\jpeg-exports
            If (LCase(Image_Extension) = "*.tif" Or LCase(Image_Extension) = "*.bmp") Then
                Call JPEG_Conversion()
            Else
                Standard_Process()
            End If
        End If
    End Sub

    'JPGs renamed and copied to new folder; they are unmodified otherwise
    Private Sub Standard_Process()
        Dim i As Integer
        Dim LP_image_number As Integer
        Dim Check_image_number As Integer
        Dim foundFile As String
        Dim number_code As String
        Dim new_name As String

        i = 1
        For Each foundFile In My.Computer.FileSystem.GetFiles(Image_Path, Microsoft.VisualBasic.FileIO.SearchOption.SearchTopLevelOnly, "*.jpg")
            If i < 10 Then
                number_code = "0" + CStr(i)
            Else
                number_code = CStr(i)
            End If

            new_name = RTI_Directory & "jpeg-exports\" & txtObjectID.Text & "_" & number_code & ".jpg"
            My.Computer.FileSystem.CopyFile(foundFile, new_name)
            i += 1
        Next

        LP_image_number = CInt(lps_load(0))
        Check_image_number = LP_image_number

        Dim lpsWriter As New System.IO.StreamWriter(RTI_Directory & txtObjectID.Text & ".lp")

        lpsWriter.WriteLine(lps_load(0))

        i = 1
        For Each foundFile In My.Computer.FileSystem.GetFiles(RTI_Directory & "jpeg-exports\", Microsoft.VisualBasic.FileIO.SearchOption.SearchTopLevelOnly, "*.jpg")
            lpsWriter.WriteLine(foundFile & " " & lps_load(i))
            i += 1
        Next

        lpsWriter.Close()

        'Write metadata text file if needed
        If txtNotes.Text <> "" Then
            Dim notesWriter As New System.IO.StreamWriter(RTI_Directory & txtObjectID.Text & ".notes.txt")
            notesWriter.Write(txtNotes.Text)
            notesWriter.Close()
        End If

        Beep()

        MsgBox("Renamed and copied " & CStr(LP_image_number) & " image files into new directory and created .lp file.")

        cmdMake_Batch.Enabled = True


    End Sub
    'Image files in BMP and TIFF formats converted to JPG, saved in RTI_Directory\jpeg-exports
    Private Sub JPEG_Conversion()
        Dim i As Integer
        Dim LP_image_number As Integer
        Dim Check_image_number As Integer
        Dim foundFile As String
        Dim number_code As String
        Dim new_name As String
        Dim Conversion_image As Bitmap


        'Parameters for saving JPEG with 100% quality
        Dim jpgEncoder As ImageCodecInfo = GetEncoder(ImageFormat.Jpeg)

        ' Create an Encoder object based on the GUID 
        ' for the Quality parameter category. 
        Dim myEncoder As System.Drawing.Imaging.Encoder = System.Drawing.Imaging.Encoder.Quality

        ' Create an EncoderParameters object. 
        ' An EncoderParameters object has an array of EncoderParameter 
        ' objects. In this case, there is only one 
        ' EncoderParameter object in the array. 
        Dim myEncoderParameters As New EncoderParameters(1)
        Dim myEncoderParameter As New EncoderParameter(myEncoder, 100&)
        myEncoderParameters.Param(0) = myEncoderParameter

        i = 1
        For Each foundFile In My.Computer.FileSystem.GetFiles(Image_Path, Microsoft.VisualBasic.FileIO.SearchOption.SearchTopLevelOnly, Image_Extension)
            If i < 10 Then
                number_code = "0" + CStr(i)
            Else
                number_code = CStr(i)
            End If

            Conversion_image = Bitmap.FromFile(foundFile)
            new_name = RTI_Directory & "jpeg-exports\" & txtObjectID.Text & "_" & number_code & ".jpg"
            Conversion_image.Save(new_name, jpgEncoder, myEncoderParameters)
            Conversion_image.Dispose()
            i += 1
        Next

        LP_image_number = CInt(lps_load(0))
        Check_image_number = LP_image_number

        Dim lpsWriter As New System.IO.StreamWriter(RTI_Directory & txtObjectID.Text & ".lp")

        lpsWriter.WriteLine(lps_load(0))

        i = 1
        For Each foundFile In My.Computer.FileSystem.GetFiles(RTI_Directory & "jpeg-exports\", Microsoft.VisualBasic.FileIO.SearchOption.SearchTopLevelOnly, "*.jpg")
            lpsWriter.WriteLine(foundFile & " " & lps_load(i))
            i += 1
        Next

        lpsWriter.Close()

        'Write metadata text file if needed
        If txtNotes.Text <> "" Then
            Dim notesWriter As New System.IO.StreamWriter(RTI_Directory & txtObjectID.Text & ".notes.txt")
            notesWriter.Write(txtNotes.Text)
            notesWriter.Close()
        End If

        Beep()

        MsgBox("Converted and saved " & CStr(LP_image_number) & " *.jpg image files into new directory, and created .lp file.")
        cmdMake_Batch.Enabled = True


    End Sub

    'Initial program load
    Private Sub RTI_Processor_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        Dim j As Integer
        Dim Image_check As Integer

        'Makes it impossible to resize the window
        Me.FormBorderStyle = System.Windows.Forms.FormBorderStyle.Fixed3D
        lblFolderPath.Text = ""
        lbllpsFile.Text = ""

        If Not My.Computer.FileSystem.FileExists(App_Path() & "default.lps") Then
            MsgBox("No default.lps calibration file found; lps file is needed for processing. Select an lps file; use the menu item to set it as default if desired.")
            Call OpenLpsFile()
            'Application.Exit()
            Exit Sub
        End If

        If Not My.Computer.FileSystem.FileExists(App_Path() & "ptmfitter.exe") Then

            MsgBox("You are missing the ptmfitter.exe file required to create ptm files. The website where you can download the ptmfitter.zip file will open automatically. Until you copy the appropriate files into the RTI Processor application folder and restart the app, the PTM options will be disabled.")
            Dim webAddress As String = "http://www.hpl.hp.com/research/ptm/downloads/download.html"
            Process.Start(webAddress)
            chkPTM_LRGB.Enabled = False
            chkPTM_RGB.Enabled = False
        End If


        Dim lpsReader As New System.IO.StreamReader(App_Path() & "default.lps")
        j = 0

        Do While lpsReader.Peek() <> -1
            lps_load(j) = lpsReader.ReadLine()
            j += 1
        Loop

        lpsReader.Close()

        Image_check = CInt(lps_load(0))
        If Image_check <> j - 1 Then
            MsgBox("Number of light position lines in default.lps calibration file does not match first number in file; this needs to be resolved. Exiting program...")
            Application.Exit()
        End If

        lbllpsFile.Text = App_Path() & "default.lps"
        'Call Extract_LP_Components()
        Call Initialize()

    End Sub
    Private Sub Initialize()
    End Sub

    Private Sub cmdClear_Notes_Click(sender As Object, e As EventArgs) Handles cmdClear_Notes.Click
        Dim Answer As Integer
        Answer = MsgBox("Are you sure you want to clear the note text?", vbYesNo)
        If Answer = vbNo Then Exit Sub

        txtNotes.Text = ""



    End Sub

    Private Sub cmdExit_Click(sender As Object, e As EventArgs) Handles cmdExit.Click
        Application.Exit()
    End Sub

    Private Sub ExitToolStripMenuItem_Click(sender As Object, e As EventArgs) Handles ExitToolStripMenuItem.Click
        Application.Exit()
    End Sub

    Public Function App_Path() As String
        Return System.AppDomain.CurrentDomain.BaseDirectory()
    End Function
    'Converts an lp file (with filenames) into an lps file that only has the image count and the XYZ data from the lp file. The lps file is used to generate an lp file for
    'processing with the selected images.
    Private Sub CreateLpsFromLpToolStripMenuItem_Click(sender As Object, e As EventArgs) Handles CreateLpsFromLpToolStripMenuItem.Click
        Dim input_lp_file As String
        Dim output_lps_file As String
        Dim i, j As Integer
        Dim lp_hold(257) As String
        Dim Space_position As Long
        Dim Calibration_line As String
        Dim dlgSaveLPS As New SaveFileDialog()

        dlgSaveLPS.InitialDirectory = App_Path()
        dlgSaveLPS.Filter = "lps files (*.lps)|*.lps|All files (*.*)|*.*"

        input_lp_file = "null"

        If dlgOpenLP.ShowDialog() = Windows.Forms.DialogResult.OK Then
            input_lp_file = dlgOpenLP.FileName
        End If
        If input_lp_file = "null" Then Exit Sub

        Dim lpReader As New System.IO.StreamReader(input_lp_file) 'Specify file for input

        j = 0
        Do While lpReader.Peek() <> -1
            lp_hold(j) = lpReader.ReadLine()
            j += 1
        Loop
        lpReader.Close()

        If j - 1 <> CInt(lp_hold(0)) Then
            MsgBox("Number of calibration lines in file " & input_lp_file & " does not match first line of lp file; " & lp_hold(0) & " <> " & CStr(j) & ". Check the lp file in a text editor for problems.")
            Exit Sub
        End If



        If dlgSaveLPS.ShowDialog() = Windows.Forms.DialogResult.OK Then
            output_lps_file = dlgSaveLPS.FileName
            Dim lpWriter As New System.IO.StreamWriter(output_lps_file) 'Specify file for output
            lpWriter.WriteLine(lp_hold(0))
            For i = 1 To j - 1
                Space_position = InStr(lp_hold(i), " ")
                Calibration_line = lp_hold(i).Substring(CInt(Space_position))
                'Calibration_line = Right(lp_hold(i), Len(lp_hold(i)) - Space_position)
                lpWriter.WriteLine(Calibration_line)
            Next i
            lpWriter.Close()

        End If

    End Sub



    'Lets you select an alternate lps file
    Private Sub OpenLpsFileToolStripMenuItem_Click(sender As Object, e As EventArgs) Handles OpenLpsFileToolStripMenuItem.Click
        Dim j As Integer
        Dim Image_check As Integer
        Dim LPS_file As String
        Dim dlgLPS As New OpenFileDialog()

        dlgLPS.InitialDirectory = App_Path()
        dlgLPS.Filter = "lps files (*.lps)|*.lps|All files (*.*)|*.*"

        'My.Computer.FileSystem.CurrentDirectory = Application_Path
        'My.Computer.FileSystem.CurrentDirectory = App_Path()

        'MsgBox("Current app path" & App_Path())
        LPS_file = "null"

        If dlgLPS.ShowDialog() = Windows.Forms.DialogResult.OK Then
            LPS_file = dlgLPS.FileName
        End If
        If LPS_file = "null" Then Exit Sub

        Dim lpsChecker As New System.IO.StreamReader(LPS_file)
        j = 0

        Do While lpsChecker.Peek() <> -1
            lps_load(j) = lpsChecker.ReadLine()
            j += 1
        Loop

        Image_check = CInt(lps_load(0))
        If Image_check <> j - 1 Then
            MsgBox("Number of light position lines in default.lps calibration file does not match first number in file; this needs to be resolved. Previous light position data kept.")
            lpsChecker.Close()
            Exit Sub
        End If
        lpsChecker.Close()

        Dim lpsReader As New System.IO.StreamReader(LPS_file)

        j = 0
        Do While lpsReader.Peek() <> -1
            lps_load(j) = lpsReader.ReadLine()
            j += 1
        Loop

        lpsReader.Close()

        lbllpsFile.Text = LPS_file

        'Call Extract_LP_Components()

    End Sub

    Private Sub OpenLpsFile()
        Dim j As Integer
        Dim Image_check As Integer
        Dim LPS_file As String

        My.Computer.FileSystem.CurrentDirectory = App_Path()

        LPS_file = "null"

        If dlgLpsload.ShowDialog() = Windows.Forms.DialogResult.OK Then
            LPS_file = dlgLpsload.FileName
        End If
        If LPS_file = "null" Then
            MsgBox("lps file not loaded successfully; exiting application...")
            Application.Exit()
        End If

        Dim lpsChecker As New System.IO.StreamReader(LPS_file)
        j = 0

        Do While lpsChecker.Peek() <> -1
            lps_load(j) = lpsChecker.ReadLine()
            j += 1
        Loop

        Image_check = CInt(lps_load(0))
        If Image_check <> j - 1 Then
            MsgBox("Number of light position lines in default.lps calibration file does not match first number in file; this needs to be resolved. Exiting application...")
            lpsChecker.Close()
            Application.Exit()
            Exit Sub
        End If
        lpsChecker.Close()

        Dim lpsReader As New System.IO.StreamReader(LPS_file)

        j = 0
        Do While lpsReader.Peek() <> -1
            lps_load(j) = lpsReader.ReadLine()
            j += 1
        Loop

        lpsReader.Close()

        lbllpsFile.Text = LPS_file

        'Call Extract_LP_Components()

    End Sub
    'If you've loaded a different lps file, and want to go back to the default ...
    Private Sub ReloadDefaultLpsToolStripMenuItem_Click(sender As Object, e As EventArgs) Handles ReloadDefaultLpsToolStripMenuItem.Click
        Dim j As Integer

        Dim lpsReader As New System.IO.StreamReader(App_Path() & "default.lps")
        j = 0

        Do While lpsReader.Peek() <> -1
            lps_load(j) = lpsReader.ReadLine()
            j += 1
        Loop

        lpsReader.Close()
        lbllpsFile.Text = App_Path() & "default.lps"

        'Call Extract_LP_Components()

    End Sub
    'Saves loaded lps data as the default lps that loads automatically when the program starts
    Private Sub SaveCurrentLpsAsDefaultToolStripMenuItem_Click(sender As Object, e As EventArgs) Handles SaveCurrentLpsAsDefaultToolStripMenuItem.Click
        Dim i As Integer

        If App_Path() & "default.lps" = lbllpsFile.Text Then
            MsgBox("Default file data is currently loaded into program.")
            Exit Sub
        End If

        Dim lpsWriter As New System.IO.StreamWriter(App_Path() & "default.lps")

        For i = 0 To CInt(lps_load(0))
            lpsWriter.WriteLine(lps_load(i))
        Next

        lpsWriter.Close()

        MsgBox("New default lps file written.")

    End Sub


    'This creates the batch programs that are then run to process the RTI images into the RTI datafiles.
    'If postpone processing option is selected, commands are written both into a batch file for that dataset, and a combined "master.bat" file
    'that adds the commands to the end of currently present command set.
    Private Sub cmdMake_Batch_Click(sender As Object, e As EventArgs) Handles cmdMake_Batch.Click
        Dim Master_Exists As Boolean

        'RTI_Directory & txtObjectID.Text & ".lp"

        If Not My.Computer.FileSystem.FileExists(RTI_Directory & txtObjectID.Text & ".lp") Then
            MsgBox("There's currently no valid lp file ready for processing.")
            Exit Sub
        End If

        Master_Exists = False
        If My.Computer.FileSystem.FileExists(Storage_Directory & "master_batch.bat") Then Master_Exists = True

        If Not chkPTM_LRGB.Checked And Not chkPTM_RGB.Checked And Not chkHSH_2.Checked And Not chkHSH_3.Checked Then
            MsgBox("No output options selected.")
            Exit Sub
        End If

        Dim batchFile As New System.IO.StreamWriter(RTI_Directory & txtObjectID.Text & "_process.bat")
        Dim createMaster As New System.IO.StreamWriter(Storage_Directory & "master_batch.bat", Master_Exists)


        If chkPTM_LRGB.Checked Then
            batchFile.WriteLine("""" & App_Path() & "ptmfitter"" -LRGB -i """ & RTI_Directory & txtObjectID.Text & ".lp"" -o """ & RTI_Directory & txtObjectID.Text & "_LRGB.ptm""")
            If chkPostpone_Process.Checked Then
                createMaster.WriteLine("""" & App_Path() & "ptmfitter"" -LRGB -i """ & RTI_Directory & txtObjectID.Text & ".lp"" -o """ & RTI_Directory & txtObjectID.Text & "_LRGB.ptm""")
            End If
        End If

        If chkPTM_RGB.Checked Then
            batchFile.WriteLine("""" & App_Path() & "ptmfitter"" -RGB -i """ & RTI_Directory & txtObjectID.Text & ".lp"" -o """ & RTI_Directory & txtObjectID.Text & "_RGB.ptm""")
            If chkPostpone_Process.Checked Then
                createMaster.WriteLine("""" & App_Path() & "ptmfitter"" -RGB -i """ & RTI_Directory & txtObjectID.Text & ".lp"" -o """ & RTI_Directory & txtObjectID.Text & "_RGB.ptm""")
            End If
        End If

        If chkHSH_2.Checked Then
            batchFile.WriteLine("""" & App_Path() & "hshfitter"" """ & RTI_Directory & txtObjectID.Text & ".lp"" 2 """ & RTI_Directory & txtObjectID.Text & "_order_2.rti""")
            If chkPostpone_Process.Checked Then
                createMaster.WriteLine("""" & App_Path() & "hshfitter"" """ & RTI_Directory & txtObjectID.Text & ".lp"" 2 """ & RTI_Directory & txtObjectID.Text & "_order_2.rti""")
            End If
        End If

        If chkHSH_3.Checked Then
            batchFile.WriteLine("""" & App_Path() & "hshfitter"" """ & RTI_Directory & txtObjectID.Text & ".lp"" 3 """ & RTI_Directory & txtObjectID.Text & "_order_3.rti""")
            If chkPostpone_Process.Checked Then
                createMaster.WriteLine("""" & App_Path() & "hshfitter"" """ & RTI_Directory & txtObjectID.Text & ".lp"" 3 """ & RTI_Directory & txtObjectID.Text & "_order_3.rti""")
            End If
        End If

        batchFile.WriteLine("del """ & RTI_Directory & "temp.hsh""")
        batchFile.WriteLine("del """ & Image_Path & "temp.hsh""")
        batchFile.WriteLine("del """ & Storage_Directory & "temp.hsh""")
        batchFile.WriteLine("del """ & App_Path() & "temp.hsh""")
        batchFile.Close()

        If chkPostpone_Process.Checked Then
            createMaster.WriteLine("del """ & RTI_Directory & "temp.hsh""")
            createMaster.WriteLine("del """ & Image_Path & "temp.hsh""")
            createMaster.WriteLine("del """ & Storage_Directory & "temp.hsh""")
            createMaster.WriteLine("del """ & App_Path() & "temp.hsh""")
        End If
        createMaster.Close()

        If Master_Exists = False And chkPostpone_Process.CheckState = CheckState.Unchecked Then
            My.Computer.FileSystem.DeleteFile(Storage_Directory & "master_batch.bat")
        End If

        If chkPostpone_Process.Checked Then
            MsgBox("Batch file " & txtObjectID.Text & "_process.bat created; run this file, or the master_process.bat file to complete processing.")
        Else
            Shell(RTI_Directory & txtObjectID.Text & "_process.bat")
            If First_load = 0 Then
                MsgBox("Generating RTI output files (process visible in command window); you can prepare another data run while this one is running.")
                First_load = 1
            End If
        End If

    End Sub

    'Saves loaded TIF and BMP files as JPGs for processing through RTI fitters

    'http://support.microsoft.com/kb/324788
    Private Sub VaryQualityLevel(Image_data As Bitmap, Save_path As String)
        ' Get a bitmap. 
        Dim jpgEncoder As ImageCodecInfo = GetEncoder(ImageFormat.Jpeg)

        ' Create an Encoder object based on the GUID 
        ' for the Quality parameter category. 
        Dim myEncoder As System.Drawing.Imaging.Encoder = System.Drawing.Imaging.Encoder.Quality

        ' Create an EncoderParameters object. 
        ' An EncoderParameters object has an array of EncoderParameter 
        ' objects. In this case, there is only one 
        ' EncoderParameter object in the array. 
        Dim myEncoderParameters As New EncoderParameters(1)

        Dim myEncoderParameter As New EncoderParameter(myEncoder, 100&)
        myEncoderParameters.Param(0) = myEncoderParameter
        Image_data.Save("C:\test.jpg", jpgEncoder, myEncoderParameters)
        Return

    End Sub 'VaryQualityLevel

    Private Function GetEncoder(ByVal format As ImageFormat) As ImageCodecInfo

        Dim codecs As ImageCodecInfo() = ImageCodecInfo.GetImageDecoders()

        Dim codec As ImageCodecInfo
        For Each codec In codecs
            If codec.FormatID = format.Guid Then
                Return codec
            End If
        Next codec
        Return Nothing

    End Function
End Class



